/************************************************************************/
/*																		*/
/*	main.c	--	Analog IO
 *      Reading from analog input
 *      Writing to PWM output
 * 
 * Based on FreeRTOS V7.3.0 - Copyright (C) 2012 Real Time Engineers Ltd.
 *
 *
 * G Mason, Mechanical Engineering Seattle University
 * Revised 2013 05 06
/*																		*/
/************************************************************************/


#include "RTOS/FreeRTOS.h"
#include "RTOS/task.h"
#include "RTOS/queue.h"
#include "RTOS/timers.h"

#include "MX4cKsetup.h"
#include "MX4cKsettings.h"

// configuration for analog input
// configure to automatically read AN1
// See Microchip documentation Section 17 and
// PIC32 Peripherial Library Guide Section 18
#define CONFIG1 (ADC_MODULE_ON | ADC_FORMAT_INTG32 | ADC_CLK_AUTO | ADC_AUTO_SAMPLING_ON)
#define CONFIG2 (ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_ON | ADC_SAMPLES_PER_INT_1 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF)
#define CONFIG3 (ADC_CONV_CLK_INTERNAL_RC | ADC_SAMPLE_TIME_15)
#define CONFIGPORT (ENABLE_AN1_ANA)
#define CONFIGSCAN (SKIP_SCAN_AN0  | SKIP_SCAN_AN2 | SKIP_SCAN_AN3 | SKIP_SCAN_AN4 | SKIP_SCAN_AN5 | SKIP_SCAN_AN6 | SKIP_SCAN_AN7 | SKIP_SCAN_AN8 | SKIP_SCAN_AN9 | SKIP_SCAN_AN10 | SKIP_SCAN_AN11 | SKIP_SCAN_AN12 | SKIP_SCAN_AN13 | SKIP_SCAN_AN14| SKIP_SCAN_AN15)



// forward declaration
static void myTimer(xTimerHandle xTimer);
static void SetupHardware(void);
static void myTask(void *pvParameters);

/*
 * Create the  tasks then start the scheduler.
 */
int main(void) {
    xTimerHandle xTimer1;

    // Prepare the hardware to run 
    SetupHardware();

    // Configure built-in hardware.  Only one built-in LED is used in this example, so this configuration could be simplified
    // configure the built-in LEDs
    //   LED1 is on RB10
    //	 LED2 is on RB11
    //	 LED3 is on RB12
    //	 LED4 is on RB13
    //Set ports for onboard LEDs to outputs
    PORTSetPinsDigitalOut(IOPORT_B, BIT_10 | BIT_11 | BIT_12 | BIT_13);
    PORTClearBits(IOPORT_B, BIT_10 | BIT_11 | BIT_12 | BIT_13);
    // set the built-in buttons as inputs
    PORTSetPinsDigitalIn(IOPORT_A, BIT_6 | BIT_7);


    // configure the analog input
    // see defines above
    // this setup is for manually polling the ADC.  If higher data
    // collection rate is needed can configure the ADC differently.
    CloseADC10();

    SetChanADC10(ADC_CH0_NEG_SAMPLEA_NVREF);
    OpenADC10(CONFIG1, CONFIG2, CONFIG3, CONFIGPORT, CONFIGSCAN);
    // Setup for the ADC10.
    EnableADC10(); // Enables the ADC10.


    // setup PWM
    CloseOC1(); // make sure everything is clear

    // set port for open drain
    PORTSetPinsDigitalOut(IOPORT_D, BIT_1); // just being careful
    // mPORTDOpenDrainOpen(BIT_1);

    // use 10000 pulsed from a 10MHz peripherial clock = 10KHz PWM frequency
    // can vary duty from 0 to 1000
    OpenTimer2(T2_ON | T2_PS_1_1, 1000);

    // Enable Output Compare  this toggles an output based on the relationship between a counter and two count variables
    // configuration =  32 bit Mode  | Timer2 is used for comparistion signal | Continuous O/P   | OC Pin High , S Compare value, Compare value
    OpenOC2(OC_ON | OC_TIMER_MODE32 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH, 1000, 0);

    // set the point where the PWM signal changes
    SetDCOC2PWM(5000); // change every 5000 pulses of timer2, since there are 10000 pulses per cycle, this give 50% duty cycle




    // create a timer to flash one of the built-in LEDs.  This lets us know the program is running.
    // this timer function is called every 500 ticks (the RTOS it configured for 1ms / tick
    // timers are configured with RTOS priority 2  (priority 0 is lowest (least important))

    xTimer1 = xTimerCreate((const signed char *) "Flasher", // A text name, purely to help debugging.
            (500), // The timer period in tick counts  (1000 * 1ms = 1sec)
            pdTRUE, // This is an auto-reload timer, so xAutoReload is set to pdTRUE.
            (void *) 0, // The ID is not used, so can be set to anything.
            myTimer // The callback function that inspects the status of all the other tasks.
            );

    // make sure the timer was created then start it
    if (xTimer1 != NULL) {
        xTimerStart(xTimer1, 0);
    }


    // create a thread
    xTaskCreate(myTask, // The function that implements the task.
            (signed char *) "myTask", // The text name assigned to the task - for debug only as it is not used by the kernel.
            configMINIMAL_STACK_SIZE+0, // The size of the stack to allocate to the task.
                                        // add to this size to account for local variable
                                        // or function call stack usage
            (void *) (0), // The parameter passed to the task - just to check the functionality.
            1, // The priority assigned to the task. Currently lower than the timer
            NULL); // The task handle is not required, so NULL is passed.


    /* Start the tasks and timer running. */
    vTaskStartScheduler();

    for (;;);
}

/*-----------------------------------------------------------*/
static void myTimer(xTimerHandle xTimer) {
    // Flash an LED to show something is happening
    // This function toggles one of the built in LEDs.
    PORTToggleBits(IOPORT_B, BIT_10);
}

/*-----------------------------------------------------------*/
static void myTask(void *pvParameters) {
    // a simple task.  Once called the task loop run indefinitely only
    // being interrupted by higher priority tasks

    // useful functions to control timing in a task
    // portTickType xTaskGetTickCount();             // return current system tick count
    //
    // vTaskDelay( portTickType xTicksToDelay );     // block for fix number of ticks
    //
    // vTaskDelayUntil( portTickType *pxPreviousWakeTime,  portTickType xTimeIncrement );
    //        block untils a specific time.  Use this for repeating tasks when you need to compensate for computational delay

    int ain;
    int pwmduty; // between 0 and 999 for our device
    for (;;) {

        ain = ReadADC10(0);
        // this is a 10 bit converter using the internal reference, Vref= 3.3V
        // V measured = ain/1023.0*Vref;

        // set the PWM output so it follows the analog input
        // calculate the duty cycle
        pwmduty = ain * 1000.0 / 1023.0;
        if (pwmduty >= 1000) pwmduty = 999;
        // shouldn't need this next line unless having trouble at 0% duty cycle - this is somewhat of a hack
        if (pwmduty <= 1) pwmduty = 2;

        SetDCOC2PWM(pwmduty);
        // wait 100ms and do this again - the delay is not required, you could run this loop continuously
        // the RTOS will interrupt the loop for the timer.  This task has priority 1, the timer has priority 2
        // if you need more accurate timing using the vTaskDelayUntil function and set the task priority to 3
        vTaskDelay(100);
    }
}


//  ------------ Hardware Setup -----------------

static void SetupHardware(void) {
    /* Set the system and peripheral bus speeds and enable the program cache*/
    SYSTEMConfigPerformance(configCPU_CLOCK_HZ - 1);
    mOSCSetPBDIV(OSC_PB_DIV_8);

    /* Setup to use the external interrupt controller. */
    INTEnableSystemMultiVectoredInt();
  
    portDISABLE_INTERRUPTS();  // disable just to start

}

/*-----------------------------------------------------------
 *  Hooks for RTOS.  Are not required to modify these
 *  May want to turn on LED or some other external signal if an error
 *  occurs
 *-----------------------------------------------------------*/

void vApplicationMallocFailedHook(void) {
    // support function.  See RTOS Starter
    // if an error occurs go into an infinite loop
    taskDISABLE_INTERRUPTS();
    for (;;);
}

void vApplicationIdleHook(void) {
    // support function.  See RTOS Starter
}

void vApplicationStackOverflowHook(xTaskHandle pxTask, signed char *pcTaskName) {
    (void) pcTaskName;
    (void) pxTask;
    // support function.  See RTOS Starter
    // if an error occurs go into an infinite loop
    taskDISABLE_INTERRUPTS();
    for (;;);
}

void vApplicationTickHook(void) {
    // support function.  See RTOS Starter
}

void _general_exception_handler(unsigned long ulCause, unsigned long ulStatus) {
    // support function.  See RTOS Starter
    // if an error occurs go into an infinite loop
    for (;;);
}

